// PCMHAIR: HAIR EXAMPLES
// **********************
// This animation renders separate frames each showing the results of
// different hair option values.
//
// Render with eight frames, non-cyclic.
// Aspect ratio is 1:1 (eg. 300x300, 500x500, etc).

// CAMERA AND LIGHTS
   camera {right x up y location <0, 1, -5> look_at <0, 0, 0> angle 40}
   light_source {<-10, 20, -100> rgb 1 shadowless}
   light_source {<0, 60, 0> rgb .5}

// BACKGROUND AND FLOOR
   background {rgb 1}
   disc {-y, y, 5 pigment {rgb 1} finish {ambient .5 diffuse 1}}
   default {finish {ambient .2 diffuse .8}}

// SET HAIR OPTIONS FOR EACH FRAME
   #include "pcmhair.mcr"
   #switch (int(clock*7)+1)

// FRAME 1: SHORT FUR USING triangle_patch_hairs_object
   #case (1)
   #declare hair_length = .2;
   #declare hair_thickness = .005;
   #declare hair_rotation = x*75;
   #declare hair_sky = x;             // Causes hair to fall in -x direction
   #declare hairs_per_patch = 500;
   #declare hair_rotate_to_surface = true;
   #declare hair_objects_per_unit = 100;
   #declare hair_texture = texture {
      pigment {wrinkles color_map {[0 rgb <1, 0, 0>] [1 rgb <1, 1, 0>]} scale .6}
      finish {phong .3 phong_size 25}}
   #declare apply_hair_materials_to_mesh = true;
   triangle_patch_hairs_object ()
   #break

// FRAME 2: LONG HAIR (uses default smooth_triangle_hairs_object)
   #case (2)
   #declare hair_thickness = .005;
   #declare hair_length = 1.25;
   #declare hair_radius = .75;
   #declare hair_objects_per_unit = 80;
   #declare hair_patch_size = .1;
   #declare hairs_per_patch = 30;
   #declare hair_rotation = <40, 180, 0>;
   #declare hair_rotation_turb = <.1, .2, 0>;
   #declare hair_pattern_pigment = pigment {bozo color_map {[0 rgb <.5, .25, .1>] [1 rgb <.6, .4, .2>]} scale .01}
   #declare hair_texture = texture {finish {specular .3 roughness .6}}
   #declare apply_hair_materials_to_mesh = true;
   #break

// FRAME 3: SPIKED HAIR/FUR USING triangle_patch_hairs_object
   #case (3)
   #declare hair_length = .3;
   #declare hair_thickness = .03;
   #declare hairs_per_patch = 40;
   #declare hair_objects_per_unit = 70;
   #declare hair_patch_turb = .5;
   #declare hair_rotate_to_surface = true;
   #declare hair_texture = texture {pigment {gradient y color_map {[0 rgb <1, .5, .6>] [1 rgb <.5, .7, 1>]} translate -y*.5 scale 2.5}}
   #declare apply_hair_materials_to_mesh = true;
   triangle_patch_hairs_object ()
   #break

// FRAME 4: GRASS
   #case (4)
   #declare hair_length = .5;
   #declare hair_radius = .5;
   #declare hair_thickness = .01;
   #declare hair_rotation = <-30, 5, 0>;
   #declare hairs_per_patch = 5;
   #declare hair_patch_size = .1;
   #declare hair_objects_per_unit = 75;
   #declare hair_texture = texture {
      normal {bumps .7 scale <.05, 1, .05>}
      finish {phong .3 phong_size 15}}
   #declare hair_pattern_pigment = pigment {bozo color_map {[0 rgb <.5, .5, .2>] [1 rgb <.1, .4, .2>]} scale .01}
   #declare apply_hair_materials_to_mesh = true;
   #break

// FRAME 5: FUR TUFTS USING triangle_cluster_hairs_object
   #case (5)
   #declare hair_length = .2;
   #declare hairs_per_patch = 100;
   #declare hair_thickness = .01;
   #declare hair_objects_per_unit = 100;
   #declare hair_pattern_pigment = pigment {bozo color_map {[.3 rgb .1] [.5 rgb .9]} scale <.25, 5, .25>}
   #declare apply_hair_materials_to_mesh = true;
   triangle_cluster_hairs_object()
   #break

// FRAME 6: SEASHELLS
   #case (6)
   #declare hair_arc = .15;
   #declare hair_thickness = .3;
   #declare hair_objects_per_unit = 60;
   #declare hair_rotation = <-45, 0, 0>;
   #declare hair_rotation_turb = <.1, .1, .25>;
   #declare apply_hair_materials_to_mesh = true;
   clipped_sphere_hair_object()
   #declare hair_object = object {hair_object
      normal {radial .2 scallop_wave frequency 25 rotate x * 90}
      finish {specular .15 roughness .1 irid {.4 thickness .5}}
      scale .65}
   #declare hair_pattern_pigment = pigment {bozo
      color_map {[0 rgb <.3, .6, .8>] [1 rgb <.1, .2, .4>]}
      scale .1}
   #break

// FRAME 7: DEW DROPS/BUBBLES
   #case (7)
   #declare hair_object = sphere {0, .035 hollow}
   #declare hair_material = material {
      texture {pigment {rgbf <1, 1, 1, .8>} finish {specular .8 roughness .002 irid {.4 thickness .3}}}
      interior {ior 1.3}}
   #declare hair_objects_per_unit = 40;
   #declare hair_size_turb = 2;
   #declare mesh_textures = array[1] {texture {pigment {rgb <.6, .1, .5>}}}
   #break

// FRAME 8: MULTIPLE NUMBERS (using hair_objects array)
   #case (8)
   #declare hair_objects_per_unit = 30;
   #declare hair_objects = array[3] {
      text {ttf "Arial", "1", .2, 0 translate <-.25, 0, -.1> scale .3},
      text {ttf "Arial", "2", .2, 0 translate <-.25, 0, -.1> scale .3},
      text {ttf "Arial", "3", .2, 0 translate <-.25, 0, -.1> scale .3}}
   #declare hair_pattern_pigment = pigment {hexagon rgb <1, .95, .5>, rgb <1, .98, .6>, rgb <1, .8, .4> scale .3}
   #declare hair_rotate_to_surface = true;
   #declare mesh_texture = texture {pigment {rgb <1, .9, .5>}}
   #end

// CREATE CURRENT FRAME USING OPTIONS SET
   read_mesh ("sphere.pcm")
