/*************************************************************************
    COMPRESSED MESH MORPHING MACRO FILE FOR PERSISTENCE OF VISION 3.1
**************************************************************************

Created by Chris Colefax, 22 January 1999

Updated 13 May 1999: Added support for compressed bicubic patch surfaces;
   Removed triangle_morph_meshes () macro.

Updated 20 May 1999: Updated for Warp's Mesh Compressor 1.2a

Updated 28 Dec 1999: Updated for Warp's Mesh Compressor 2.1

NOTE: This file requires that PCM.MCR be installed

*************************************************************************/

#ifndef (read_mesh) #include "pcm.mcr" #end

#macro morph_meshes (MeshFileList, MorphPos)

// CHECK OPTIONS AND ASSIGN DEFAULTS
// *********************************
   #ifndef (debug_progress) #local debug_progress = false; #end
   #ifndef (declare_only) #local declare_only = false; #end
   #ifndef (max_mesh_count) #local max_mesh_count = 100; #end
   #ifndef (apply_mesh_textures) #local apply_mesh_textures = true; #end
   #ifdef (morph_mesh_textures) #local TexCount = dimension_size(morph_mesh_textures, 2); #else #local TexCount = 0; #end

// CREATE SINGLE MESH IF NO MORPHING REQUIRED
// ******************************************
   #local MorphCount = dimension_size(MeshFileList, 1)-1; #local SingleMesh = -1;
   #if (MorphCount = 0) #local SingleMesh = 0; #warning "PCMMORPH.MCR: You should specify more than 1 mesh file!\n" #else
      #local Time = mod(abs(MorphPos)-1e-5, MorphCount)+1e-5; #local MorphIndex = int(Time);
      #if (Time = MorphIndex) #local SingleMesh = MorphIndex; #else
      #if (Time = MorphIndex+1) #local SingleMesh = MorphIndex+1; #end #end #end

   #if (MorphCount > 0 & SingleMesh > -1 & (defined(deform_morph_meshes) | defined(create_morph_meshes)))
      #local SingleMesh = -1; #if (MorphIndex > 0) #local MorphIndex = MorphIndex - 1; #end #end
  
   #if (SingleMesh > -1)
      #if (apply_mesh_textures & TexCount > 0) #local mesh_textures = array[TexCount] #local C = 0; #while (C < TexCount)
         #local mesh_textures[C] = texture {morph_mesh_textures[SingleMesh][C]}
      #local C = C+1; #end #end
      read_mesh (MeshFileList[SingleMesh])
   #else

// MORPH TWO SELECTED MESH FILES
// *****************************
   #local FromFile = MeshFileList[MorphIndex] #local ToFile = MeshFileList[MorphIndex+1]
   #if (declare_only) #declare Mesh = array[max_mesh_count] #end
   #if (debug_progress) #debug concat("Morphing compressed mesh files \"", FromFile, "\" and \"", ToFile, "\":\n") #end
   #if (Time = MorphCount) #local Weight2 = 1; #else #local Weight2 = mod(Time, 1); #end
   #ifdef (morph_smoothing) #if (morph_smoothing > 0)
      #local Weight2 = (Weight2 < .5 ? pow(Weight2*2, morph_smoothing)*.5 : 1-pow((1-Weight2)*2, morph_smoothing)*.5);
   #end #end #local Weight1 = 1-Weight2;

   #local Version = "" #local X = 0; #local Y = 0; #local Z = 0; #local PointCount = 0; #local TriangleCount = 0; #local SmoothTriangleCount = 0; #local TriCount2 = 0; #local STriCount2 = 0; #local PatchCount = 0; #local PatchCount2 = 0;
   #local P1 = 0; #local P2 = 0; #local P3 = 0; #local P4 = 0; #local P5 = 0; #local P6 = 0; #local N1 = 0; #local N2 = 0; #local N3 = 0; #local N4 = 0; #local N5 = 0; #local N6 = 0;
   #local MeshCount = 0; 

// CHECK FILE FORMATS
// ******************
   #fopen File1 FromFile read  #fopen File2 ToFile read  #while (defined(File1) & defined(File2))
   #local VerID = 0; #read (File1, Version)
      #if (strcmp(Version, "PCM1") = 0) #local VerID = 1; #end #if (strcmp(Version, "PCM2") = 0) #local VerID = 2; #end
      #if (VerID = 0) #warning concat("PCMMORPH.MCR: A mesh file you specified, \"", FromFile, "\", is in an unsupported format!\n") #fclose File1 #end
      #ifdef (mesh_texture) #local Texture1 = mesh_texture #undef mesh_texture #end
   #local VerID2 = 0; #read (File2, Version)
      #if (strcmp(Version, "PCM1") = 0) #local VerID2 = 1; #end #if (strcmp(Version, "PCM2") = 0) #local VerID2 = 2; #end
      #if (VerID2 = 0) #warning concat("PCMMORPH.MCR: A mesh file you specified, \"", ToFile, "\", is in an unsupported format!\n") #fclose File2 #end
      #ifdef (mesh_texture) #local Texture2 = mesh_texture #undef mesh_texture #end
   #if (defined(File1) & defined(File2)) #if (VerID != VerID2) #warning ("PCMMORPH.MCR: The mesh files you specified are not in the same formats!\n" #fclose File1 #fclose File2 #end #end

// READ POINT LISTS
// ****************
   #if (defined(File1) & defined(File2))
   #ifdef (deform_morph_meshes) deform_morph_meshes () #else
      #read (File1, PointCount) #if (PointCount > 0) #local PointList1 = array[PointCount]
      #if (debug_progress) #debug concat("   Reading ", str(PointCount, 0, 0), " vertices...") #end
      #local Counter = 0; #while (Counter < PointCount) #read (File1, X, Y, Z) #local PointList1[Counter] = <X, Y, Z>; #local Counter = Counter+1; #end
      #if (debug_progress) #debug "done!\r" #end #end

      #read (File2, PointCount) #if (PointCount > 0) #local PointList2 = array[PointCount]
      #if (debug_progress) #debug concat("   Reading ", str(PointCount, 0, 0), " vertices...") #end
      #local Counter = 0; #while (Counter < PointCount) #read (File2, X, Y, Z) #local PointList2[Counter] = <X, Y, Z>; #local Counter = Counter+1; #end
      #if (debug_progress) #debug "done!\r" #end #end
   #end

// INTERPOLATE TRIANGLE LISTS TO CREATE MESH
// *****************************************
   #switch (VerID)
   #case (1) #read (File1, TriangleCount, SmoothTriangleCount) #read (File2, TriCount2, STriCount2)
      #if (TriangleCount != TriCount2 | SmoothTriangleCount != STriCount2)
         #warning concat ("PCMMORPH.MCR: The mesh files you specified, \"", FromFile, "\" and \"", ToFile, "\", are not compatible!\n") #fclose File1 #fclose File2
      #else
         #if (debug_progress) #debug concat("   Creating morphed mesh with ", str(TriangleCount, 0, 0), " triangles and ", str(SmoothTriangleCount, 0, 0), " smooth triangles...") #end
      #end
      #break
   #case (2) #read (File1, PatchCount) #read (File2, PatchCount2)
      #if (PatchCount != PatchCount2)
         #warning concat ("PCMMORPH.MCR: The mesh files you specified, \"", FromFile, "\" and \"", ToFile, "\", are not compatible!\n") #fclose File1 #fclose File2
      #else
         #if (debug_progress) #debug concat("   Creating morphed mesh with ", str(PatchCount, 0, 0), " bicubic patches...") #end
      #end
   #end

   #if (defined(File1) & defined(File2))
      #if (declare_only) #declare Mesh[MeshCount] = #end
      #ifdef (create_morph_meshes) object {create_morph_meshes () #else PCM_morph_surfaces () #end

// APPLY MESH TEXTURES
// *******************
   #if (apply_mesh_textures)
      #if (TexCount > 0) texture {average texture_map {[Weight1 morph_mesh_textures[MorphIndex][mod(MeshCount, TexCount)]] [Weight2 morph_mesh_textures[MorphIndex+1][mod(MeshCount, TexCount)]]}} #else
      #ifdef (mesh_textures) texture {mesh_textures[mod(MeshCount, dimension_size(mesh_textures, 1))]} #else
      #if (defined(Texture1) & defined(Texture2)) texture {average texture_map {[Weight1 Texture1] [Weight2 Texture2]}} #end #end #end #end
      }
   #local MeshCount = MeshCount+1; #if (debug_progress) #debug "done!\n" #end
   #end #end #end

   #if (declare_only & MeshCount > 0)
      #debug "Created morphed Mesh[0] " #if (MeshCount > 1) #debug concat("to Mesh[", str(MeshCount-1, 0, 0), "] ") #end #debug concat("from file \"", FromFile, "\" to \"", ToFile, "\"\n")
   #else
      #if (debug_progress) #debug concat("Morphed ", str(MeshCount, 0, 0), " mesh(es) from file \"", FromFile, "\" to \"", ToFile, "\"\n") #end
   #end #end
#end

// DEFAULT SURFACE INTERPOLATION/CREATION MACROS
// *********************************************
#macro PCM_morph_surfaces ()
   #switch (VerID)
   #case (1) mesh {
      #local Counter = 0; #while (Counter < TriangleCount)
         #read (File1, P1, P2, P3) #read (File2, P4, P5, P6)
         triangle {PointList1[P1]*Weight1+PointList2[P4]*Weight2, PointList1[P2]*Weight1+PointList2[P5]*Weight2, PointList1[P3]*Weight1+PointList2[P6]*Weight2}
      #local Counter = Counter+1; #end
      #local Counter = 0; #while (Counter < SmoothTriangleCount)
         #read (File1, P1, N1, P2, N2, P3, N3) #read (File2, P4, N4, P5, N5, P6, N6)
         smooth_triangle {PointList1[P1]*Weight1+PointList2[P4]*Weight2, PointList1[N1]*Weight1+PointList2[N4]*Weight2, PointList1[P2]*Weight1+PointList2[P5]*Weight2, PointList1[N2]*Weight1+PointList2[N5]*Weight2, PointList1[P3]*Weight1+PointList2[P6]*Weight2, PointList1[N3]*Weight1+PointList2[N6]*Weight2}
      #local Counter = Counter+1; #end
      #break

   #case (2) union {
      #local Counter = 0; #while (Counter < PatchCount)
         PCM_read_morph_patch_options () bicubic_patch {type PT flatness PF u_steps PU v_steps PV
         #local Counter2 = 0; #while (Counter2 < 16) #read (File1, P1) #read (File2, P2)
            PointList1[P1]*Weight1+PointList2[P2]*Weight2 #if (Counter2 < 15) , #end
         #local Counter2 = Counter2+1; #end } 
      #local Counter = Counter+1; #end
   #end
#end

#macro PCM_read_morph_patch_options ()
   #read (File1, P1) #if (P1 != -1) #declare PT = P1; #read (File1, PF, PU, PV) #end
   #read (File2, P1) #if (P1 != -1) #declare PT = P1; #read (File2, PF, PU, PV) #end
   PCM_override_patch_options ()
#end

// MORPH MESH CREATION MACROS
// **************************
#macro object_morph_meshes (SingleObject)
   union { #local Counter = 0; #switch (VerID)
   #case (1) #while (Counter < TriangleCount+SmoothTriangleCount)
      #if (Counter < TriangleCount) #read (File1, P1, P2, P3) #read (File2, P4, P5, P6)
      #else #read (File1, P1, N1, P2, N2, P3, N3) #read (File2, P4, N4, P5, N5, P6, N6) #end
      object {SingleObject translate (Weight1*(PointList1[P1]+PointList1[P2]+PointList1[P3])+Weight2*(PointList2[P4]+PointList2[P5]+PointList2[P6]))/3}
   #local Counter = Counter+1; #end #break

   #case (2) #local PList = array[16] #while (Counter < PatchCount)
      PCM_read_morph_patch_options ()
      #declare Counter2 = 0; #while (Counter2 < 16) #read (File1, P1) #read (File2, P2) #local PList[Counter2] = PointList1[P1]*Weight1+PointList2[P2]*Weight2; #declare Counter2 = Counter2+1; #end
      #local U = .5; #while (U < PU) #local V = .5; #while (V < PV) object {SingleObject translate PCM_get_patch_point (PList, U/PU, V/PV)} #local V = V+1; #end #local U = U+1; #end
   #local Counter = Counter+1; #end #end }
#end
